#!/usr/bin/perl -w

# get_php_fpm_status.pl gets runtime status of a php-cgi server (pool)
#
# Copyright (c) 2011 Oleksandr Moskalenko <alexander.moskalenko@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

use strict;
use FCGI::Client;
use IO::Socket::INET;
use JSON -support_by_pp;
use Getopt::Long;

# command line parameters with some defaults

my $host = '';         # server host
my $port = 9999;       # tcp port
my $script = '/status';       # test script (absolute path starting at / - root directory -)
my $timeout = 3;       # timeout in seconds

# check command line options

GetOptions ('H=s' => \$host, 'p=i' => \$port, 's=s' => \$script, 't=i' => \$timeout);

if (($host eq '') || ($port eq '')) {
   print "Usage: check_php-cgi.pl -H host -p port [-s <test script>] [-t <timeout seconds>]\n";
   exit(-1);
}

# run check

my $sock = IO::Socket::INET->new(
    PeerAddr => $host,
    PeerPort => $port,
    Timeout  => $timeout,
    Proto    => 'tcp',
) or die $!;

my $client = FCGI::Client::Connection->new( sock => $sock );
my ( $stdout, $stderr ) = $client->request(
    +{
	    SCRIPT_FILENAME => $script,
	    QUERY_STRING    => 'json',
	    REQUEST_METHOD  => 'GET',
	    SCRIPT_NAME     => $script,
    },
    ''
);

my ($headers,$content) = split "\n\r?\n", $stdout;
my $json = new JSON;
my $json_text = $json->allow_nonref->utf8->relaxed->escape_slash->loose->allow_singlequote->allow_barekey->decode($content);

print 'accepted:' . $json_text->{"accepted conn"} . ' idle:' . $json_text->{"idle processes"} . ' active:'. $json_text->{"active processes"} . ' total:' . $json_text->{"total processes"} . "\n";
